          subroutine A1_wf(q0,qv,kt1,ks1,p_m,kt2,ks2,p_r,kt3,ks3,kj,
     &                                     wf1_re,wf1_im,md,ifl,irs)
******************************************************************
* inputs 
* q0 - virtual photon energy
* qv - virtual photon momentum
* kti - (i=1,2,3) isospins(1->proton,-1->neutron)
* ks1,ks2,ks3 - spin projection of 1,2,3 nucleon -1-> -1/2, 1-> 1/2
* kj - spin projection of he3 -1-> -1/2, 1-> 1/2
* idr - Flag of He3 WF (1,2 -> initialisation, 0->computation)
* md - method number in PV term, see  subroutine A1_z
* ifl - =1->pole case,= 2->PV case, 0 -> both
* irs = 22 pp rescattering, 21 pn rescattering
*       second digit indicates the isospin of the rescattered nucleon
* irs = 121 - np case - knocked-out neutron with rescattering off proton-r
* irs = 122 - np case - knocked-out neutron with rescattering off proton-s    
*
*inputs through common 
*/par/pi,pm,pmp,pmn,tm   : pi amd masses(constants)
*******************************************************************
        external phl, phu, un_1_re, un_1_im
        dimension p_m(3),p_r(3)
        common/par/pi,pm,pmp,pmn,tm,eb
	common/proton_num/zpro	
	common/vphoton/Q2,q0c,qvc
	common/knocked/  p_f_x,  p_f_y,  p_f_z
        common/missing/  p_m_x,  p_m_y,  p_m_z
        common/recoil_r/ p_r_x,  p_r_y,  p_r_z
        common/recoil_s/ p_s_x,  p_s_y,  p_s_z
        common/conf/it1,is1,it2,is2,it3,is3,ij
        common/iflag/iflag
        common/enginv/s,its
        common/delta0/delta_r,delta_s
        common/pvstaff/accuracy,downlimit,uplimit 
        common/method/method
        common/emiss/ems
        its    = irs
        iflag  = ifl
        method = md
        it1    = kt1
        is1    = ks1
        it2    = kt2
        is2    = ks2
        it3    = kt3
        is3    = ks3
        ij     = kj
        accuracy  =  0.1
        downlimit = -1.0
        uplimit   =  1.0
        Q2  = qv**2 - q0**2
        qvc = qv
        q0c = q0

        p_m_x =  p_m(1)
        p_m_y =  p_m(2)
        p_m_z =  p_m(3)

        p_m_v = sqrt(p_m_x**2 + p_m_y**2 + p_m_z**2)
        t_2m = sqrt(4.0*pm**2 + p_m_v**2)-2.0*pm

        p_r_x =  p_r(1)
        p_r_y =  p_r(2)
        p_r_z =  p_r(3)

        p_s_x = -(p_m_x + p_r_x)  
        p_s_y = -(p_m_y + p_r_y) 
        p_s_z = -(p_m_z + p_r_z) 

        p_f_x = p_m_x
        p_f_y = p_m_y
        p_f_z = p_m_z + qv

        p_f = sqrt(p_f_x**2 + p_f_y**2 + p_f_z**2)
        pr  = sqrt(p_r_x**2 + p_r_y**2 + p_r_z**2)
        ps  = sqrt(p_s_x**2 + p_s_y**2 + p_s_z**2)


        E_f = sqrt(pmp**2+p_f**2)
        E_r = sqrt(pmp**2+pr**2)
        E_s = sqrt(pmn**2+ps**2)
        T_r = E_r - pm
        T_s = E_s - pmn
        delta_r = q0/qv*(T_r + T_s + eb) 
     &          +(2.*pm*(T_r + T_s) + p_m_v**2 - (T_r+T_s)**2)/(2.0*qv)

*        delta_r = q0/qv*(ems+t_2m) 
*     &          + (2.*pm*(ems+t_2m) + p_m_v**2 - (ems+t_2m)**2)/(4.0*qv)
        delta_s = delta_r
**************************************************
* This block calculates the s of the NN system
**************************************************
        if(irs.eq.22)then          ! pp case
        s = 2*(pmp**2+E_f*E_r - p_f_x*p_r_x -p_f_y*p_r_y -p_f_z*p_r_z)
        elseif(irs.eq.21)then     ! pn case
        s = pmp**2+pmn**2+2.0*(E_f*E_s - 
     &                p_f_x*p_s_x -p_f_y*p_s_y -p_f_z*p_s_z) 
        endif

        if(irs.gt.23)then          ! np case
        E_f = sqrt(pmn**2+p_f**2)
        E_r = sqrt(pmp**2+pr**2)
        E_s = sqrt(pmp**2+ps**2)
        T_r = E_r - pmp
        T_s = E_s - pmp
        delta_r = q0/qv*(T_r + T_s + eb)
     &          + (2.*pm*(T_r + T_s) + p_m_v**2 - (T_r+T_s)**2)/(2.0*qv)

*        delta_r = q0/qv*(ems+t_2m) 
*     &          + (2.*pm*(ems+t_2m) + p_m_v**2  - (ems+t_2m)**2)/(4.0*qv)

        delta_s = delta_r
        if(irs.eq.121)then ! neutron rescatters off proton -r
        s = pmp**2+pmn**2+2.0*(E_f*E_r - 
     &                p_f_x*p_r_x -p_f_y*p_r_y -p_f_z*p_r_z)
        elseif(irs.eq.122)then ! neutron rescatters off proton -s
        s = pmp**2+pmn**2+2.0*(E_f*E_s - 
     &                p_f_x*p_s_x -p_f_y*p_s_y -p_f_z*p_s_z)
        endif
        endif
        
***************************************************************

        
        eps = 0.20
        qa  = 0.0
        qb  = 1.
        sum_re = 0.0
        call gadap2(qa,qb,phl,phu,un_1_re,eps,sum_re) 
        eps = 0.20
        qa  = 0.0
        qb  = 1.
        sum_im = 0.0
  	call gadap2(qa,qb,phl,phu,un_1_im,eps,sum_im)                

        wf1_re = sum_re
        wf1_im = sum_im         

        return
        end

        function phl(q)
	phl = 0.0
	return
	end
	function phu(q)
	phu = 2.0*acos(-1.0)
	return
	end

	function  un_1_re(q,phiq)
        dimension pv1(3), pv2(3)
        common/par/pi,pm,pmp,pmn,tm,eb
	common/proton_num/zpro	
	common/vphoton/Q2,q0,qv
        common/missing/  p_m_x,  p_m_y,  p_m_z
        common/recoil_r/ p_r_x,  p_r_y,  p_r_z
        common/knocked/  p_f_x,  p_f_y,  p_f_z
 
        common/delta0/ delta_r, delta_s
        common/conf/kt1,ks1,kt2,ks2,kt3,ks3,kj
 	common/enginv/s,its
        common/iflag/iflag !iflag =1->pole case,= 2->PV case, 0 -> both
        common/pvstaff/accuracy,downlimit,uplimit 
        common/method/method
        un_1_re = 0.0
        
	q_x      = q*cos(phiq)
	q_y      = q*sin(phiq)

 	pv1(3)   = p_m_z + delta_r
	pv1(2)   = p_m_y + q_y
	pv1(1)   = p_m_x + q_x
	p_1     = sqrt(pv1(1)**2+pv1(2)**2+pv1(3)**2)
*        write(6,*)delta_r,delta_s

                    ireg = 21
*       if(ktf.eq.-1)ireg = 12
       sigma0   = 1.0 !sigma_en(p_m_x,p_m_y,p_m_z,ireg)
       sigma1   =  1.0 !sigma_en(pv1(1),pv1(2),pv1(3),ireg)
       rsg = sqrt(sigma1/sigma0)
        
        if (its.eq.22.or.its.eq.121) then
	pv2(3)   = p_r_z - delta_r
        pv2(2)   = p_r_y - q_y
	pv2(1)   = p_r_x - q_x
        elseif(its.eq.21.or.its.eq.122) then
	pv2(3)   = p_r_z 
        pv2(2)   = p_r_y 
	pv2(1)   = p_r_x 
        endif
	p_2     = sqrt(pv2(1)**2+pv2(2)**2+pv2(3)**2)
        E_2     = sqrt(pm**2 + p_2**2)

****************************************************
* Calculation of t_on and t_off
****************************************************
****************************************************
*        Third nucleon 
****************************************************
        pv3_x = -(pv1(1) + pv2(1))
        pv3_y = -(pv1(2) + pv2(2))
        pv3_z = -(pv1(3) + pv2(3))
        pv3 = sqrt(pv3_x**2 + pv3_y**2 + pv3_z**2)
        E_3 = sqrt(pm**2 + pv3**2)
 
	p_f = sqrt(p_f_x**2 + p_f_y**2 + p_f_z**2)
	e_f = sqrt(pm**2 + p_f**2)

	p_tf_x   = pv1(1) 
	p_tf_y   = pv1(2)
	p_tf_z   = pv1(3) + qv
	p_tf     = sqrt(p_tf_x**2 + p_tf_y**2 + p_tf_z**2) 
	e_tf_on  = sqrt(pm**2 + p_tf**2)
	e_tf_off = tm - E_2 - E_3 + q0

	delta_pf2 = (p_f_x-p_tf_x)**2 + (p_f_y-p_tf_y)**2
     &                                + (p_f_z-p_tf_z)**2

  	t_on  = (e_f - e_tf_on)**2  - delta_pf2
	t_off = (e_f - e_tf_off)**2 - delta_pf2



        un_1_re_pole = 0.0
        un_1_re_pv   = 0.0
        
        if (iflag.eq.1.or.iflag.eq.0) then
************************************************************************
*       Pole term
************************************************************************
        wfre = 0.0
        wfim = 0.0
           
        if(zpro.gt.1.5)then  ! He3
	call wf_he3_nb(kt1,ks1,pv1,kt2,ks2,pv2,kt3,ks3,kj,wfre,wfim,0) 
        else !H3
	call wf_h3_nb(kt1,ks1,pv1,kt2,ks2,pv2,kt3,ks3,kj,wfre,wfim,0) 
        endif

        call f_NN_on(s,t_on,f_NN_on_re,f_NN_on_im,its)
              
        un_1_re_pole = -(f_NN_on_im*wfre + f_NN_on_re*wfim)
        endif
        
        if  (iflag.eq.2.or.iflag.eq.0) then   
************************************************************************
*    PV term
************************************************************************
        a1_z_re = 0.0
        a1_z_im = 0.0
        call A1_z(method,accuracy,uplimit,q_x,q_y,a1_z_re,a1_z_im)
        call  f_NN_off(s,t_off,f_NN_off_re,f_NN_off_im,its)        

        un_1_re_pv= -(f_NN_off_re*a1_z_re - f_NN_off_im*a1_z_im)
************************************************************************
        endif
        
        un_1_re =rsg*(1.0/4.0)*q*(un_1_re_pole + un_1_re_pv)/(2.0*pi)**2 

        return
	end


        function  un_1_im(q,phiq)
        dimension pv1(3), pv2(3)
        common/par/pi,pm,pmp,pmn,tm,eb
	common/proton_num/zpro	
	common/vphoton/Q2,q0,qv
        common/missing/  p_m_x,  p_m_y,  p_m_z
        common/recoil_r/ p_r_x,  p_r_y,  p_r_z
        common/knocked/  p_f_x,  p_f_y,  p_f_z
        common/delta0/ delta_r,delta_s
        common/conf/kt1,ks1,kt2,ks2,kt3,ks3,kj
	common/enginv/s,its
        common/iflag/iflag
        common/pvstaff/accuracy,downlimit,uplimit 
        common/method/method
        un_1_im = 0.0
	q_x      = q*cos(phiq)
	q_y      = q*sin(phiq)
	
        pv1(3)   = p_m_z + delta_r
	pv1(2)   = p_m_y + q_y
	pv1(1)   = p_m_x + q_x
	p_1     = sqrt(pv1(1)**2+pv1(2)**2+pv1(3)**2)
        

                   ireg = 21
*       if(ktf.eq.-1)ireg = 12
       sigma0   = 1.0 !sigma_en(p_m_x,p_m_y,p_m_z,ireg)
       sigma1   =  1.0 !sigma_en(pv1(1),pv1(2),pv1(3),ireg)
       rsg = sqrt(sigma1/sigma0)

        if (its.eq.22.or.its.eq.121) then
	pv2(3)   = p_r_z - delta_r
        pv2(2)   = p_r_y - q_y
	pv2(1)   = p_r_x - q_x
        elseif(its.eq.21.or.its.eq.122) then
 	pv2(3)   = p_r_z 
        pv2(2)   = p_r_y 
	pv2(1)   = p_r_x 
        endif
	p_2     = sqrt(pv2(1)**2+pv2(2)**2+pv2(3)**2)
        E_2     = sqrt(pm**2 + p_2**2)

****************************************************
* Calculation of t_on and t_off
****************************************************
****************************************************
*        Third nucleon 
****************************************************
        pv3_x = -(pv1(1) + pv2(1))
        pv3_y = -(pv1(2) + pv2(2))
        pv3_z = -(pv1(3) + pv2(3))
        pv3 = sqrt(pv3_x**2 + pv3_y**2 + pv3_z**2)
        E_3 = sqrt(pm**2 + pv3**2)
 
	p_f = sqrt(p_f_x**2 + p_f_y**2 + p_f_z**2)
	e_f = sqrt(pm**2 + p_f**2)

	p_tf_x   = pv1(1) 
	p_tf_y   = pv1(2)
	p_tf_z   = pv1(3) + qv
	p_tf     = sqrt(p_tf_x**2 + p_tf_y**2 + p_tf_z**2) 
	e_tf_on  = sqrt(pm**2 + p_tf**2)
	e_tf_off = tm - E_2 - E_3 + q0

	delta_pf2 = (p_f_x-p_tf_x)**2 + (p_f_y-p_tf_y)**2
     &                                + (p_f_z-p_tf_z)**2

  	t_on  = (e_f - e_tf_on)**2  - delta_pf2
	t_off = (e_f - e_tf_off)**2 - delta_pf2


        
        un_1_im_pole = 0.0
        un_1_im_pv = 0.0

        if (iflag.eq.1.or.iflag.eq.0) then
************************************************************************
*       Pole term
************************************************************************
        wfre = 0.0
        wfim = 0.0

        if(zpro.gt.1.5)then
	call wf_he3_nb(kt1,ks1,pv1,kt2,ks2,pv2,kt3,ks3,
     &               kj,wfre,wfim,0)                     
        else
	call wf_h3_nb(kt1,ks1,pv1,kt2,ks2,pv2,kt3,ks3,
     &               kj,wfre,wfim,0)                     
        endif

        call  f_NN_on(s,t_on,f_NN_on_re,f_NN_on_im,its) 	
       
        un_1_im_pole = (f_NN_on_re*wfre - f_NN_on_im*wfim)
       
        endif
        if(iflag.eq.2.or.iflag.eq.0) then
************************************************************************
*    PV term
************************************************************************
        a1_z_re = 0.0
        a1_z_im = 0.0
        call A1_z(method,accuracy,uplimit,q_x,q_y,a1_z_re,a1_z_im)
        call  f_NN_off(s,t_off,f_NN_off_re,f_NN_off_im,its)        

        un_1_im_pv= -(f_NN_off_re*a1_z_im + f_NN_off_im*a1_z_re)
************************************************************************
        endif               

        un_1_im =rsg*(1.0/4.0)*q*(un_1_im_pole + un_1_im_pv)/(2.0*pi)**2        
        return
	end



        subroutine A1_z(method,accuracy,uplimit,q_x,q_y,a1_zre,a1_zim)
************************************************************
* This suboutine calculates PV integral over k_z
* In the write-up it corresponds to the function K(...)
* method = 1,2,3,4,... defines the method which is used 
*                      for PV integration
*          1 - uses "dcauch" function from CERNLIB
*          2 - uses gadap subroutine 
*          3 - uses rgquad from CERNLIB
*          4 - uses simple SIMPSON
*************************************************************
        common/par/pi,pm,pmp,pmn,tm,eb
        external func,func1
        real*8 func,da,db,ds,deps,dsum_re,dsum_im
        common/k_prp/qx,qy
        common/conf/kt1,ks1,kt2,ks2,kt3,ks3,kj
        common/ireim/ireim
        common/nnneps/nnn,eps
        a1_zre = 0.0
        a1_zim = 0.0
        if(method.eq.0)return
        eps = 0.01
        qx = q_x
        qy = q_y              
        eps = accuracy/10.

        if (method.eq.1) then
        da    = -dble(uplimit)
        db    =  dble(uplimit)
        ds    =  0.0
        deps  =  dble(accuracy)
        ireim =  1
        dsum_re = 0 !dCAUCH(func,dA,dB,dS,dEPS)
        ireim = -1
        dsum_im = 0 !dCAUCH(func,dA,dB,dS,dEPS)

        a1_zre  = -dsum_re
        a1_zim  = -dsum_im
        return        
************************* method 2***************************
        elseif (method.eq.2) then
        a0   = eps
        b0   = uplimit
        eps0 = accuracy
        ireim =  1
        call gadap(a0,b0,func1,eps0,part1_re)
        a0   = eps
        b0   = uplimit
        eps0 = accuracy
        ireim = -1
        call gadap(a0,b,func1,eps0,part1_im)

        a1_zre = -part1_re
        a1_zim = -part1_im
        return
************************ method 3*****************************        
        elseif (method.eq.3) then
        nnn = 32
        a0 = eps
        b0 = uplimit
        ireim = 1
        part1_re = 0! rgquad(func1,a0,b0,nnn)
        a0 = eps
        b0 = uplimit
        ireim = -1
        part1_im = 0! rgquad(func1,a0,b0,nnn)

        a1_zre = -part1_re
        a1_zim = -part1_im
        return
********************** end method 4*****************************        
        elseif (method.eq.4) then        
        nn = 1000
        a0 = eps
        b0 = uplimit
        ireim = 1
        call simpson(a0,b0,func1,nn,part1_re)
        a0 = eps
        b0 = uplimit
        ireim = -1
        call simpson(a0,b0,func1,nn,part1_im)
        endif

        a1_zre = -part1_re
        a1_zim = -part1_im
        return
        end


      function func(dq)
******************************************************
* this function is the integrand of "dcauch" method
******************************************************
      real *8 func,func_re,func_im,dq
      dimension pv1(3), pv2(3)
      common/delta0/delta_r,delta_s
      common/k_prp/q_x,q_y
      common/missing/  p_m_x,  p_m_y,  p_m_z
      common/recoil_r/ p_r_x,  p_r_y,  p_r_z
 
      common/conf/kt1,ks1,kt2,ks2,kt3,ks3,kj
      common/ireim/ireim
      common/enginv/s,its
      q_z = dq
      pv1(1) = p_m_x + q_x 
      pv1(2) = p_m_y + q_y
      pv1(3) = p_m_z + q_z + delta_r
      
      if (its.eq.22.or.its.eq.121) then
      pv2(1) = p_r_x - q_x 
      pv2(2) = p_r_y - q_y 
      pv2(3) = p_r_z - q_z - delta_r
      elseif(its.eq.21.or.its.eq.122) then
      pv2(1) = p_r_x  
      pv2(2) = p_r_y  
      pv2(3) = p_r_z 
      endif
      
      p1 = sqrt(pv1(1)**2 + pv1(2)**2 + pv1(3)**2)
      p2 = sqrt(pv2(1)**2 + pv2(2)**2 + pv2(3)**2)
      func = 0.0
      if (p1.le.0.005.and.p2.le.0.005) return
      idrosh = 0
      call wf_he3_nb(kt1,ks1,pv1,kt2,ks2,pv2,kt3,ks3,kj,wfre,wfim,0)
      
      func_re = wfre/q_z 
      func_im = wfim/q_z
      
      if(ireim.eq.1 )func = func_re
      if(ireim.eq.-1)func = func_im
      return
      end 



      function func1(q)
*******************************************************************
* this function is the integrand of "gadap" and "rgquad" method
*******************************************************************
      dimension pv1(3), pv2(3), pvn1(3), pvn2(3)
      common/par/pi,pm,pmp,pmn,tm,eb
      common/k_prp/q_x,q_y
      common/missing/  p_m_x,  p_m_y,  p_m_z
      common/recoil_r/ p_r_x,  p_r_y,  p_r_z
      common/conf/kt1,ks1,kt2,ks2,kt3,ks3,kj
      common/ireim/ireim
      common/delta0/delta_r,delta_s
      common/enginv/s,its
      q_z = q
      pv1(1)  = p_m_x + q_x
      pv1(2)  = p_m_y + q_y
      pv1(3)  = p_m_z + q_z + delta_r
      pvn1(1) = pv1(1)
      pvn1(2) = pv1(2)
      pvn1(3) = p_m_z - q_z + delta_r
      if (its.eq.22.or.its.eq.121.or.its.eq.12) then 
      pv2(1)  = p_r_x - q_x 
      pv2(2)  = p_r_y - q_y 
      pv2(3)  = p_r_z - q_z - delta_r
      pvn2(1) = pv2(1)
      pvn2(2) = pv2(2)
      pvn2(3) = p_r_z + q_z - delta_r
      elseif(its.eq.21.or.its.eq.122.or.its.eq.13) then
      pv2(1)   = p_r_x  
      pv2(2)   = p_r_y  
      pv2(3)   = p_r_z 
      pvn2(1)  = pv2(1)
      pvn2(2)  = pv2(2)
      pvn2(3)  = p_r_z
      endif
    
      p1     = sqrt(pv1(1)**2 + pv1(2)**2 + pv1(3)**2)
      p2     = sqrt(pv2(1)**2 + pv2(2)**2 + pv2(3)**2)

      func1 = 0.0
      if (p1.le.0.005.and.p2.le.0.005) return
      call wf_he3_nb(kt1,ks1,pv1,kt2,ks2,pv2,kt3,ks3,kj,wfre,wfim,0)
      call wf_he3_nb(kt1,ks1,pvn1,kt2,ks2,pvn2,kt3,ks3,kj,wfren,wfimn,0)

      func1_re = (wfre-wfren)/q_z
      func1_im = (wfim-wfimn)/q_z
      if(ireim.eq.1 )func1 = func1_re/pi
      if(ireim.eq.-1)func1 = func1_im/pi
      return
      end 



        subroutine
     & wf_he3_nbu(mt1,ms1,pv1,mt2,ms2,pv2,mt3,ms3,kj,wfre,wfim,iflag)
***********************************************************************

* kt1,kt2,kt3 - isospin projections of 1,2,3 nucleons -1-neutron,1-proton
* ks1,ks2,ks3 - spin projection of 1,2,3 nucleon -1-> -1/2, 1-> 1/2
* kj - spint projection of he3 -1/2, 1-> 1/2
* pv1,pv2 - three vectors of nucleon 1 and 2 in GeV/c
* wfre1,wfim1,wfre2,wfim2 - re and im part of the wave function in GeV-6
*                          1 and -  total spin projections -1/2 and 1/2
*
* kj -spin projecttion of the he3
*iflag = 1,2,0
* momenta in Gev
* wave GeV^{-3}
***********************************************************************
       use getwavemod
       dimension pv1(3),pv2(3)
       CHARACTER(LEN=80) :: wavein
       COMPLEX wave
       wfre = 0.0
       wfim = 0.0
       fmg = 0.197326
       mbj =  kj
       if(iflag.eq.1.)then
       WAVEIN = 'he_3.av18+tml5156.jmax=6.esuche.cdep_app.fad.c.form'
       CALL readwave(WAVEIN,1,.false.)
       write(6,*)wavein
       return
       elseif(iflag.eq.2)then          
       CALL mbjreset(mbj,.false.)
        wfre = 0.0
        wfim = 0.0      
       return
       endif


      p1x = pv1(1)/fmg
      p1y = pv1(2)/fmg
      p1z = pv1(3)/fmg

      p2x = pv2(1)/fmg
      p2y = pv2(2)/fmg
      p2z = pv2(3)/fmg

      p3x = -(p1x+p2x)
      p3y = -(p1y+p2y)
      p3z = -(p1z+p2z)

      wave=getwave(p1x,p1y,p1z,ms1,mt1,p2x,p2y,p2z,ms2,mt2,ms3,mt3)
     X    +getwave(p2x,p2y,p2z,ms2,mt2,p3x,p3y,p3z,ms3,mt3,ms1,mt1)
     X    +getwave(p3x,p3y,p3z,ms3,mt3,p1x,p1y,p1z,ms1,mt1,ms2,mt2)
      wfre =  real(wave)/fmg**3
      wfim = aimag(wave)/fmg**3
*      write(6,*)wave
      return
      end
      
